import 'package:api/converter/request_response/calendar_event.dart';
import 'package:api/model/interface/calendar_event.dart';
import 'package:api/model/request/calendar_event.dart';
import 'package:api/model/response/calendar_event.dart';
import 'package:api/model/team.dart';
import 'package:api/model/user.dart';
import 'package:lib/lib.dart';

class CalendarEventApiEndpoint extends CheckedDbApiEndpoint<CalendarEventRequest, CalendarEventResponse> {
  final User requestingUser;
  final Iterable<Team> requestingUserTeams;

  @override
  Future<bool> Function(CalendarEventRequest) get createItemAllowed => _createItemAllowed;

  @override
  Future<bool> Function(CalendarEventResponse) get getItemAllowed => _getItemAllowed;

  @override
  Future<bool> Function(CalendarEventRequest, CalendarEventResponse) get updateItemAllowed => _updateItemAllowed;

  @override
  Future<bool> Function(CalendarEventResponse) get deleteItemAllowed => _deleteItemAllowed;

  CalendarEventApiEndpoint({
    required super.database,
    required this.requestingUser,
    required this.requestingUserTeams,
    super.itemFromRequest = CalendarEventRequestResponseConverter.fromRequest,
    super.itemToResponse = CalendarEventRequestResponseConverter.toResponseData,
  });

  Future<bool> _createItemAllowed(CalendarEventRequest item) {
    return Future.value(
      _isUserInTeam(item)
      && _isUserOwner(item)
    );
  }

  Future<bool> _getItemAllowed(CalendarEventResponse item) {
    return Future.value(
      _isUserInTeam(item)
    );
  }

  Future<bool> _updateItemAllowed(CalendarEventRequest item, CalendarEventResponse originalItem) {
    return Future.value(
      item.id == originalItem.id
      && _isUserInTeam(item)
      && _isUserOwner(item)
      && _isUserOwner(originalItem)
    );
  }

  Future<bool> _deleteItemAllowed(CalendarEventResponse item) {
    return Future.value(
      _isUserOwner(item)
    );
  }

  bool _isUserOwner(CalendarEvent item) {
    return requestingUser.id == item.userId;
  }

  bool _isUserInTeam(CalendarEvent item) {
    return requestingUserTeams.any((team) => team.id == item.teamId);
  }
}
