import 'package:api/converter/request_response/debt.dart';
import 'package:api/model/debt.dart';
import 'package:api/model/user.dart';
import 'package:api/service/debt.dart';
import 'package:lib/lib.dart';

class DebtApiEndpoint extends ApiEndpoint {
  final User requestingUser;
  final DebtService debtService;
  final Map<String, dynamic> Function(Debt) itemToResponse = DebtRequestResponseConverter.toResponseData;

  DebtApiEndpoint({required this.requestingUser, required this.debtService});

  @override
  Future<Map<String, dynamic>> handleDelete(context) {
    throw ApiActionForbiddenException("delete", "debt");
  }

  @override
  Future<Map<String, dynamic>> handleGet(context) async {
    var items = await debtService.getDebt(requestingUser.id).where((e) => checkGetAllowed(e, requestingUser.id)).toList();

    return listResponse(context, items, itemToResponse);
  }

  @override
  Future<Map<String, dynamic>> handlePost(context) {
    throw ApiActionForbiddenException("post", "debt");
  }

  @override
  Future<Map<String, dynamic>> handlePut(context) {
    throw ApiActionForbiddenException("put", "debt");
  }

  bool checkGetAllowed(Debt debt, String userId) {
    return debt.creditor == userId || debt.debtor == userId;
  }
}
