import 'package:api/converter/request_response/membership.dart';
import 'package:api/model/membership.dart';
import 'package:api/model/user.dart';
import 'package:lib/lib.dart';

class MembershipApiEndpoint extends ApiEndpoint {
  static const String teamIdFieldName = "teamId";
  static const String userIdFieldName = "userId";

  final User requestingUser;
  final Stream<Membership> Function(String) getMembershipsByUserId;
  final Stream<Membership> Function(String) getMembershipsByTeamId;
  final Map<String, dynamic> Function(Membership) itemToResponse = MembershipRequestResponseConverter.toResponseData;

  MembershipApiEndpoint({
    required this.requestingUser,
    required this.getMembershipsByUserId,
    required this.getMembershipsByTeamId,
  });

  @override
  Future<Map<String, dynamic>> handleDelete(context) {
    throw ApiActionForbiddenException("delete", "membership");
  }

  @override
  Future<Map<String, dynamic>> handleGet(context) async {
    Stream<Membership> memberships;
    if(ApiHelper.hasBodyField(context, teamIdFieldName)) {
      var requestingUserTeamIds = await getMembershipsByUserId(requestingUser.id).map((e) => e.team.id).toList();
      var teamId = ApiHelper.fieldFromRequestBody<String>(context, teamIdFieldName);
      memberships = getMembershipsByTeamId(teamId).where((e) => requestingUserTeamIds.contains(e.team.id));
    }
    else {
      var userId = ApiHelper.fieldFromRequestBody<String>(context, userIdFieldName);
      if(userId != requestingUser.id) {
        throw UnauthorizedException();
      }

      memberships = getMembershipsByUserId(userId);
    }

    return listResponse(context, await memberships.toList(), itemToResponse);
  }

  @override
  Future<Map<String, dynamic>> handlePost(context) {
    throw ApiActionForbiddenException("post", "membership");
  }

  @override
  Future<Map<String, dynamic>> handlePut(context) {
    throw ApiActionForbiddenException("put", "membership");
  }
}
