import 'dart:async';

import 'package:api/converter/request_response/payment/equally_distributed.dart';
import 'package:api/model/interface/payment/equally_distributed.dart';
import 'package:api/model/request/payment/equally_distributed.dart';
import 'package:api/model/response/payment/equally_distributed.dart';
import 'package:api/model/team.dart';
import 'package:api/model/user.dart';
import 'package:lib/lib.dart';

class EquallyDistributedApiEndpoint extends CheckedDbApiEndpoint<EquallyDistributedPaymentRequest, EquallyDistributedPaymentResponse> {
  final User requestingUser;
  final Iterable<Team> requestingUserTeams;

  @override
  Future<bool> Function(EquallyDistributedPaymentRequest) get createItemAllowed => _createItemAllowed;

  @override
  Future<bool> Function(EquallyDistributedPaymentResponse) get getItemAllowed => _getItemAllowed;

  @override
  Future<bool> Function(EquallyDistributedPaymentRequest, EquallyDistributedPaymentResponse) get updateItemAllowed => _updateItemAllowed;

  @override
  Future<bool> Function(EquallyDistributedPaymentResponse) get deleteItemAllowed => _deleteItemAllowed;
  
  EquallyDistributedApiEndpoint({
    required super.database,
    required this.requestingUser,
    required this.requestingUserTeams,
    super.itemFromRequest = EquallyDistributedPaymentRequestResponseConverter.fromRequest,
    super.itemToResponse = EquallyDistributedPaymentRequestResponseConverter.toResponseData,
  });

  Future<bool> _createItemAllowed(EquallyDistributedPaymentRequest item) {
    return Future.value(
      _isUserOwner(item)
      && _isUserInTeam(item)
      && _isAmountOk(item)
    );
  }

  Future<bool> _getItemAllowed(EquallyDistributedPaymentResponse item) {
    return Future.value(
      _isUserInTeam(item)
    );
  }

  Future<bool> _updateItemAllowed(EquallyDistributedPaymentRequest item, EquallyDistributedPaymentResponse originalItem) {
    return Future.value(
      item.id == originalItem.id
      && _isUserOwner(item)
      && _isUserOwner(originalItem)
      && _isUserInTeam(item)
      && _isAmountOk(item)
    );
  }

  Future<bool> _deleteItemAllowed(EquallyDistributedPaymentResponse item) {
    return Future.value(
      _isUserOwner(item)
    );
  }

  bool _isUserOwner(EquallyDistributedPayment item) {
    return item.paidBy == requestingUser.id;
  }

  bool _isUserInTeam(EquallyDistributedPayment item) {
    return requestingUserTeams.any((team) => team.id == item.teamId);
  }

  bool _isAmountOk(EquallyDistributedPayment item) {
    return item.amount >= 0;
  }
}
