import 'package:api/converter/request_response/payment/user_to_user.dart';
import 'package:api/model/interface/payment/user_to_user.dart';
import 'package:api/model/request/payment/user_to_user.dart';
import 'package:api/model/response/payment/user_to_user.dart';
import 'package:api/model/user.dart';
import 'package:lib/lib.dart';

class UserToUserPaymentApiEndpoint extends CheckedDbApiEndpoint<UserToUserPaymentRequest, UserToUserPaymentResponse> {
  final User requestingUser;

  @override
  Future<bool> Function(UserToUserPaymentRequest) get createItemAllowed => _createItemAllowed;

  @override
  Future<bool> Function(UserToUserPaymentResponse) get getItemAllowed => _getItemAllowed;

  @override
  Future<bool> Function(UserToUserPaymentRequest, UserToUserPaymentResponse) get updateItemAllowed => _updateItemAllowed;

  @override
  Future<bool> Function(UserToUserPaymentResponse) get deleteItemAllowed => _deleteItemAllowed;

  UserToUserPaymentApiEndpoint({
    required super.database,
    required this.requestingUser,
    super.itemFromRequest = UserToUserPaymentRequestResponseConverter.fromRequest,
    super.itemToResponse = UserToUserPaymentRequestResponseConverter.toResponseData,
  });

  Future<bool> _createItemAllowed(UserToUserPaymentRequest item) {
    return Future.value(
      _isUserPayer(item)
      && _isPayerNotPayee(item)
      && _isAmountOk(item)
    );
  }

  Future<bool> _getItemAllowed(UserToUserPaymentResponse item) {
    return Future.value(
      _isUserPayer(item)
      || _isUserPayee(item)
    );
  }

  Future<bool> _updateItemAllowed(UserToUserPaymentRequest item, UserToUserPaymentResponse originalItem) {
    return Future.value(
      item.id == originalItem.id
      && _isUserPayer(item)
      && _isUserPayer(originalItem)
      && _isPayerNotPayee(item)
      && _isAmountOk(item)
    );
  }

  Future<bool> _deleteItemAllowed(UserToUserPaymentResponse item) {
    return Future.value(
      _isUserPayer(item)
    );
  }

  bool _isUserPayer(UserToUserPayment item) {
    return item.paidBy == requestingUser.id;
  }

  bool _isUserPayee(UserToUserPayment item) {
    return item.paidTo == requestingUser.id;
  }

  bool _isPayerNotPayee(UserToUserPayment item) {
    return item.paidBy != item.paidTo;
  }

  bool _isAmountOk(UserToUserPayment item) {
    return item.amount >= 0;
  }
}
