import 'package:api/converter/request_response/route.dart';
import 'package:api/model/interface/route.dart';
import 'package:api/model/request/route.dart';
import 'package:api/model/response/route.dart';
import 'package:api/model/team.dart';
import 'package:api/model/user.dart';
import 'package:lib/lib.dart';

class RouteApiEndpoint extends CheckedDbApiEndpoint<RouteRequest, RouteResponse> {
  final User requestingUser;
  final Iterable<Team> requestingUserTeams;

  @override
  Future<bool> Function(RouteRequest) get createItemAllowed => _createItemAllowed;

  @override
  Future<bool> Function(RouteResponse) get getItemAllowed => _getItemAllowed;

  @override
  Future<bool> Function(RouteRequest, RouteResponse) get updateItemAllowed => _updateItemAllowed;

  @override
  Future<bool> Function(RouteResponse) get deleteItemAllowed => _deleteItemAllowed;

  RouteApiEndpoint({
    required super.database,
    required this.requestingUser,
    required this.requestingUserTeams,
    super.itemFromRequest = RouteRequestResponseConverter.fromRequest,
    super.itemToResponse = RouteRequestResponseConverter.toResponseData,
  });

  Future<bool> _createItemAllowed(RouteRequest item) {
    return Future.value(
      _isUserInTeam(item)
      && _isUserOwner(item)
    );
  }

  Future<bool> _getItemAllowed(RouteResponse item) {
    return Future.value(
      _isUserOwner(item)
    );
  }

  Future<bool> _updateItemAllowed(RouteRequest item, RouteResponse originalItem) {
    return Future.value(
      item.id == originalItem.id
      && _isUserInTeam(item)
      && _isUserOwner(item)
      && _isUserOwner(originalItem)
    );
  }

  Future<bool> _deleteItemAllowed(RouteResponse item) {
    return Future.value(
      _isUserOwner(item)
    );
  }

  bool _isUserInTeam(Route item) {
    return item.teamId != null ? requestingUserTeams.any((e) => e.id == item.teamId) : true;
  }

  bool _isUserOwner(Route item) {
    return item.userId == requestingUser.id;
  }
}
