import 'package:api/converter/request_response/team.dart';
import 'package:api/model/team.dart';
import 'package:api/model/user.dart';
import 'package:lib/lib.dart';

class TeamApiEndpoint extends ApiEndpoint {
  final User requestingUser;
  final Future<Team> Function(String) getTeamById;
  final Future<bool> Function(User, Team) isUserInTeam;
  final Map<String, dynamic> Function(Team) itemToResponse = TeamRequestResponseConverter.toResponseData;

  TeamApiEndpoint({required this.requestingUser, required this.getTeamById, required this.isUserInTeam});

  @override
  Future<Map<String, dynamic>> handleDelete(context) {
    throw ApiActionForbiddenException("delete", "membership");
  }

  @override
  Future<Map<String, dynamic>> handleGet(context) async {
    var teamId = teamIdFromContext(context);
    var requestedTeam = await getTeamById(teamId);
    if (!await isUserInTeam(requestingUser, requestedTeam)) {
      throw UnauthorizedException();
    }

    return context.res.json(itemToResponse(requestedTeam));
  }

  @override
  Future<Map<String, dynamic>> handlePost(context) {
    throw ApiActionForbiddenException("post", "membership");
  }

  @override
  Future<Map<String, dynamic>> handlePut(context) {
    throw ApiActionForbiddenException("put", "membership");
  }

  String teamIdFromContext(final context) {
    return ApiHelper.fieldFromRequestBody(context, "teamId");
  }
}
