import 'package:api/converter/db_converter/point.dart';
import 'package:api/model/request/point.dart';
import 'package:api/model/response/point.dart';
import 'package:dart_appwrite/dart_appwrite.dart';
import 'package:lib/lib.dart';

class PointDatabase extends BaseDatabase<PointRequest, PointResponse> {
  @override
  final String databaseId = "65fddef523b653ce0952";

  @override
  final String collectionId = "65fddefc7f63cb43b15b";

  PointDatabase({
    required super.host,
    required super.projectId,
    required super.apiKey,
    super.itemToDbMap = PointDbConverter.toDbData,
    super.itemFromDbMap = PointDbConverter.fromDbData,
  });

  @override
  Future<PointResponse> create(PointRequest item) async {
    var queries = List.of([
      Query.equal(PointDbConverter.latitudeDbFieldName, item.latitude),
      Query.equal(PointDbConverter.longitudeDbFieldName, item.longitude),
    ]);
    
    var equalItems = await getFromQueries(queries).toList();
    if (equalItems.isNotEmpty) {
      return equalItems.first;
    }
    return super.create(item);
  }

  @override
  Future<PointResponse> update(PointRequest item) {
    throw DbActionForbiddenException("update", "point");
  }

  @override
  Future<PointResponse> delete(String id) {
    throw DbActionForbiddenException("delete", "point"); // Maybe deletion is possible
  }
}
