import 'dart:async';

import 'package:api/model/membership.dart';
import 'package:api/service/team_service.dart';
import 'package:api/service/user_service.dart';
import 'package:dart_appwrite/dart_appwrite.dart';
import 'package:lib/lib.dart';

class MembershipService extends AppwriteClient {
  final UserService userService;
  final TeamService teamService;

  late final Users _users;
  late final Teams _teams;

  MembershipService({required super.host, required super.projectId, required super.apiKey, required this.userService, required this.teamService}) {
    _users = Users(client);
    _teams = Teams(client);
  }

  Stream<Membership> getUserMemberships(String userId) async* {
    var user = await userService.getUser(userId);
    var memberships = (await _users.listMemberships(userId: user.id)).memberships;

    for (var membership in memberships) {
      yield Membership(
        joinedAt: DateTime.parse(membership.joined),
        team: await teamService.getTeam(membership.teamId),
        user: user,
      );
    }
  }

  Stream<Membership> getTeamMemberships(String teamId) async* {
    var team = await teamService.getTeam(teamId);
    var memberships = (await _teams.listMemberships(teamId: team.id)).memberships;

    for (var membership in memberships) {
      yield Membership(
        joinedAt: DateTime.parse(membership.joined),
        team: team,
        user: await userService.getUser(membership.userId),
      );
    }
  }
}
