import 'package:lib/src/utils/map_helper.dart';
import 'package:lib/src/utils/permission.dart';
import 'package:lib/src/utils/variables.dart';

class Permissions {
  final List<Permission> _permissions;

  List<String> get dbPermissions => _permissions.map((e) => e.dbPermissionString).toList();

  Permissions({List<Permission>? permissions}) :
    _permissions = permissions ?? [];

  Permissions add(Permission permission) {
    _permissions.add(permission);
    return this;
  }

  Permissions addFromPermissionString(String permissionString) {
    _permissions.add(Permission.parse(permissionString));
    return this;
  }

  bool isUserPermitted(String userId, String action, List<String>? userTeamIds) {
    return _permissions.any((permission) => action == permission.action && permission.isUserPermitted(userId, userTeamIds));
  }

  factory Permissions.fromDbData(Map<String, dynamic> map) {
    var dbPermissions = getRequiredMapItem<Iterable>(DatabaseVariables.permissionsColumnName, map);

    return Permissions(
      permissions: dbPermissions.map((e) => Permission.parse(e)).toList(),
    );
  }
}
