import 'dart:io';

import 'package:lib/lib.dart';

class RequestHandler<T extends DbModel> {
  static const String requestMethodGet = "GET";
  static const String requestMethodPost = "POST";
  static const String requestMethodPut = "PUT";
  static const String requestMethodDelete = "DELETE";

  final ApiEndpoint apiEndpoint;

  RequestHandler({required this.apiEndpoint});

  Future<Map<String, dynamic>> handleRequest(final context) async {
    var requestMethod = context.req.method;
    switch (requestMethod) {
      case requestMethodGet:
        return apiEndpoint.handleGet(context);

      case requestMethodPost:
        return apiEndpoint.handlePost(context);

      case requestMethodPut:
        return apiEndpoint.handlePut(context);

      case requestMethodDelete:
        return apiEndpoint.handleDelete(context);

      default:
        var message = "Method ${context.req.method} not implemented";
        context.error(message);
        throw HttpException(message, uri: Uri.parse(context.req.url));
    }
  }
}
